/*
Copyright (c) 2008-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description: Demonstrates standard state pattern of publishing a byte-array property 
*/



/**
 @file
*/

#include "publish.h"

LOCAL_D CConsoleBase* console;

/**
Publishes the byte-array property
@param aProperty The property to be published
@param aBuf The descriptor containing the byte-array values
*/
void PublishL(RProperty& aProperty,TDes16& aBuf)
	{
	console->Printf(KTxtPublish);

	// Publish the property
	// Failure to publish values in standard state publishing pattern is treated as a serious error
	User::LeaveIfError(aProperty.Set(aBuf));
	console->Printf(KTxtArray);

	// Print the value of the byte-array
	TInt bufLength = aBuf.Length();
	for(TInt ix = 0; ix < bufLength; ix++)
		{
		console->Printf(KTxtArrayElement,aBuf[ix]);
		}
	console->Printf(KTxtNewLine);
	}

void DoExampleL()
	{
	console->Printf(KTxtStPublish);
	console->Printf(KTxtDefine);

	// Define and pre-allocate memory to the byte-array property
	RProperty::Define(KMyPropertyCat, KMyPropertyName,RProperty::EByteArray,KAllowAllPolicy,KAllowAllPolicy,KArraySize);
	RProperty myProperty;
	// Create a handle to the property
	User::LeaveIfError(myProperty.Attach(KMyPropertyCat,KMyPropertyName,EOwnerThread));
	console->Printf(KTxtCreateHandle);

	// Byte-Array property value to be published
	TUint16 array[KArraySize] = {10,20,30,40};

	// Descriptor that the array contents are copied into
	TBuf16<KBufSize> buf;

	// Copy array contents into the descriptor
	buf.Copy(&array[0],sizeof(array));
	buf.SetLength(KArraySize);

	// Publish the property
	PublishL(myProperty,buf);

	// Change the array contents
	array[0] = 0x00ab;
	array[1] = 0x00bc;
	array[2] = 0x00cd;
	array[3] = 0x00de;
	array[4] = 0x00ef;
	array[5] = 0x00fa;

	buf.Copy(&array[0],sizeof(array));
	// Copy array contents into the descriptor
	buf.SetLength(KArraySize);

	// Wait for a key press
	console->Printf(KTxtEnter);
	while(console->Getch()!= EKeyEnter)
		{
		console->Printf(KTxtEnter);
		}

	// Publish the property
	PublishL(myProperty,buf);

	// Indicate the publisher will not publish any more values
	// The subscriber stops its subscription to the property when this value is published
	buf.Copy(KStop);
	console->Printf(KTxtEnter);

	// Wait for a key press
	while(console->Getch()!= EKeyEnter)
		{
		console->Printf(KTxtEnter);
		}

	// Publish the property
	PublishL(myProperty,buf);

	// Close the handle to the property

	myProperty.Close();

	// Wait for a key press
	while(console->Getch()!= EKeyEnter)
		{
		console->Printf(KTxtEnter);
		}

	// The publisher has finished so delete the property
	User::LeaveIfError(RProperty::Delete(KMyPropertyCat, KMyPropertyName));
	}

GLDEF_C TInt E32Main()
	{
	__UHEAP_MARK;
	CTrapCleanup* cleanup = CTrapCleanup::New();

	TRAPD(createError, console = Console::NewL(KTextConsoleTitle, TSize(KConsFullScreen,KConsFullScreen)));
	if (createError)
		return createError;

	TRAPD(mainError, DoExampleL());
	if (mainError)
		console->Printf(KTextFailed, mainError);
	console->Printf(KTextPressAnyKey);
	console->Getch();

	delete console;
	delete cleanup;
	__UHEAP_MARKEND;
	return KErrNone;
	}
